function [fixptval] = flt2fp(floatval,wsize,bp)
%FLT2FP Convert a floating point values to fixed point values.
%       FIXPTVAL=FLT2FP(FLOATVAL,WSIZE,BP) converts the double precision 
%       floating point vector, FLOATVAL, into an Nx2 matrix of fixed-point 
%       values, FIXPTVAL, where N is the length of the FLOATVAL vector.  
%
%       The word size of the fixed-point result is equal to abs(WSIZE). 
%       Valid WSIZE settings, are 8, 16, 32, -8, -16, or -32.  A positive
%       WSIZE specifies that FIXPTVAL consists of unsigned values, a 
%       negative WSIZE specifies that FIXPTVAL consists of two's complement 
%       values.
%
%       The binary point location of the result is specified by BP.  The
%       binary point can range from 0 to abs(WSIZE).  If you specify BP as -1,
%       then the binary point will be located such that the fixed-point value
%       has maximum precision.
%
%       The first column FIXPTVAL is the fixed-point value.  The second column 
%       of FIXPTVAL is the encoded data type.
%
%       The encoded data type (edt) which is returned in column two of FIXPTVAL
%       will be a 16-bit unsigned quantity with the following bit settings:
%
%                       15 | 14 | 13 | 12 | 11 | 10 |  9 |  8 |
%                      ----------------------------------------
%          high byte = | 0    A   W1   W0    C    R    S |  F | 
%                      ----------------------------------------
%
%                F     = 1 if float, 0 if fixed-point
%                S     = 0 if unsigned, 1 if two's complement data type
%                R     = 1 to perform rounding on result
%                C     = 1 to check ranges on result
%                W0,W1 = word size=  0: 8-bits, 1: 16-bits, 2: 32-bits
%                A     = 0 if low byte is binary point location, 1 if we
%                        are to auto-locate the resulting binary point.
%
%                Note, the fixed-point value(s) returned by this function
%                will always have the F, R, and C bits equal to 0.
%
%          low byte  = Binary point location from left (only if A == 0).

% Copyright (C) 1994,1995, The MathWorks, Inc.
% All Rights Reserved.
% $Revision: 1.1 $  $Date: 1996/04/17 20:20:03 $
%
% File       : flt2fp.m
% Edit level : 1
%
% Abstract:
%       floating-point to fixed-point conversion for use with Fixed-Point 
%       Blockset.
% 
% Modifications:
% -----------------------------------------------------------------------------
% el-1         12-16-1994      J.E. Ciolfi
%       Creation.
%

% -------------------
% | Check arguments |
% -------------------
msg = nargchk(3,3,nargin); if ~isempty(msg) error(msg); end;

if isstr(floatval) | ~isreal(floatval) | min(size(floatval)) ~= 1,
  error('Specify floatval as a vector');
end

msg = 'Valid word sizes (wsize) are: 8, 16, 32, -8, -16, -32';
if isstr(wsize) | ~isreal(wsize) | max(size(wsize)) ~= 1,
  error(msg);
end
word_size = abs(wsize);
if word_size ~= 8 & word_size ~= 16 & word_size ~= 32,
  error(msg);
end

msg = 'Valid binary point, bp, locations are -1, 0:abs(wsize)';
if isstr(bp) | ~isreal(bp) | max(size(bp)) ~= 1,
  error(msg);
end

if (bp ~= round(bp)) | (bp < -1) | (bp > word_size),
  error(msg);
end

% ------------------
% | Compute result |
% ------------------

%edt_float_bit_loc	= 8;
edt_signed_bit_loc	= 9;
%edt_round_bit_loc	= 10;
%edt_check_bit_loc	= 11;
edt_wsize_bit_loc0      = 12;
edt_wsize_bit_loc1      = 13;

%edt_float_bit_val       = 2^edt_float_bit_loc;
edt_signed_bit_val      = 2^edt_signed_bit_loc;
%edt_round_bit_val       = 2^edt_round_bit_loc;
%edt_check_bit_val       = 2^edt_check_bit_loc;
edt_wsize8_bit_val      = 0;
edt_wsize16_bit_val     = 2^edt_wsize_bit_loc0;
edt_wsize32_bit_val     = 2^edt_wsize_bit_loc1;

if word_size == 8,  wsize_bits = edt_wsize8_bit_val;  end;
if word_size == 16, wsize_bits = edt_wsize16_bit_val; end;
if word_size == 32, wsize_bits = edt_wsize32_bit_val; end;

if wsize < 0,
  signed = 1;
else
  signed = 0;
end

if bp == -1,
  min_bp = 0;
  max_bp = word_size;
else
  min_bp = bp;
  max_bp = bp;
end

if signed
  lowlimit  = -2^(word_size-min_bp-1);
  highlimit = 2^(word_size-min_bp-1) - 2^(-min_bp);
else
  lowlimit  = 0;
  highlimit = 2^(word_size-min_bp) - 2^(-min_bp);
end

n     = max(size(floatval));
fixptval = ones(n,2);


if max(floatval) > highlimit | min(floatval) < lowlimit,
  error('Magnitude of value(s) exceeds fixed-point data type range');
end


for i = 1:n,
  flt = floatval(i);

  binary_pt_loc = bp;

  if (binary_pt_loc == -1),    % Do we need to find binary point location?
    ranges = ones(word_size+1,1);
    if signed % two's complement
      if flt < 0,
        for bploc = 0:word_size, ranges(bploc+1) = -2^(word_size-bploc-1); end
	for bploc = word_size:-1:0,
          if (binary_pt_loc == -1) & (flt >= ranges(bploc+1)), 
            binary_pt_loc = bploc; 
          end
        end
      else
        for bploc = 0:word_size, 
          ranges(bploc+1) = 2^(word_size-bploc-1) - 2^(-bploc);
        end
        for bploc = word_size:-1:0,
          if (binary_pt_loc == -1) & (flt <= ranges(bploc+1)),
             binary_pt_loc = bploc; 
          end
        end
      end
    else % unsgined
      for bploc = 0:word_size, 
        ranges(bploc+1) = 2^(word_size-bploc) - 2^(-bploc);
      end
      for bploc = word_size:-1:0,
        if (binary_pt_loc == -1) & (flt <= ranges(bploc+1)),
          binary_pt_loc = bploc;
        end
      end
    end
  end

  if flt < 0,
    fixptval(i,1) = round(flt * 2^binary_pt_loc + 2^word_size);
  else
    fixptval(i,1) = round(flt * 2^binary_pt_loc);
  end

  if signed,
    fixptval(i,2) = binary_pt_loc + edt_signed_bit_val + wsize_bits;
  else
    fixptval(i,2) = binary_pt_loc + wsize_bits;
  end
end


% [EOF flt2fp.m]
